/************************************************************************
 * Spooky - a service framework for the Internet
 * ConfigFileDAO.java
 *
 * Copyright (C) 2001 Erik Karlsson.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the 
 * License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *************************************************************************/
package spooky.config;

import java.io.Serializable;
import java.util.*;

// Namespaces required for XML manipulation
import org.w3c.dom.*;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.SAXException;

import java.io.InputStream;

import javax.swing.tree.*;
import javax.swing.*;


/**
 *
 * @author  Erik Karlsson
 * @version 
 */
public class ConfigFileDAO {

    /**
     * Loads a configuration XML file from the stream. 
     * 
     * @fileStream the stream to the XML file
     * @return a XML DOM built of the file stream.
     */
    public static Document loadFile( java.io.InputStream fileStream ) throws ConfigurationLoadException{
    
        try {
        
            InputSource fileInput = new InputSource( fileStream );
            
            DocumentBuilderFactory documentFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder theBuilder = documentFactory.newDocumentBuilder();
            
            Document doc = theBuilder.parse( fileInput );
            
            return doc;
            
        }catch( SAXParseException e ) {
            throw new ConfigurationLoadException( "loadFile::saxp:: "+e.toString() );
        }catch( SAXException ee ) {
            throw new ConfigurationLoadException( "loadFile::saxe:: " +ee.toString() );
        }catch( Exception eee ) {
            throw new ConfigurationLoadException( "loadFile::exc:: " + eee.toString() );
        }
    }
    
    /**
     * Method parses the whole node from the configuration tree and builds a tree
     * from them.
     *
     * @param rootElement the XML DOM element (which name is node)
     */
    private static DefaultMutableTreeNode parseNodesFromTree( Element rootElement, String path  ) throws ConfigurationLoadException{
    
        System.err.println("ConfigFileDAO::parseTree()");
        
        ConfigurationNode theRootConfigNode = loadConfigurationNode( rootElement, "" );
        String unitName = theRootConfigNode.getNodeName();
        path += "/"+unitName;
        theRootConfigNode.setPath( path );
        
        DefaultMutableTreeNode theTreeRootNode = new DefaultMutableTreeNode( theRootConfigNode );
        
        NodeList childNodes = ((Node)rootElement).getChildNodes();
        int size = childNodes.getLength();
        
        for( int i = 0; i < size; i++ )
        {
            Node theNode = childNodes.item(i);
            if( theNode.getNodeType() == Node.ELEMENT_NODE )
            {
                Element theElement = (Element)theNode;
                String nodeName = theElement.getTagName();
                if( nodeName.equals("node") )
                {
                    DefaultMutableTreeNode newTreeNode = parseNodesFromTree( theElement, path );
                    theTreeRootNode.add( newTreeNode );
                }
            }              
        }
         System.err.println("ConfigFileDAO::parseTree()::return");
        return theTreeRootNode;
    }
   
    /**
     * Loads the default configurations. The configurations
     * are stored to instance of a <code>ConfigurationNode</code> class.
     *
     */
    public static ConfigurationNode loadDefaultNode( Document doc ) throws ConfigurationLoadException {
    
        try 
        {
            Node rootNode = (Node)doc.getDocumentElement();
            
            NodeList childNodes = rootNode.getChildNodes();
            int size = childNodes.getLength();
            ConfigurationNode theReturnNode = null;
            
            for( int i = 0; i <size; i++ )
            {
                Node theNode = childNodes.item(i);
                if( theNode.getNodeType() == Node.ELEMENT_NODE )
                {
                    Element theElement = (Element)theNode;
                    if( theElement.getTagName().equals("default-node") )
                    {
                        theReturnNode = loadConfigurationNode( theElement, "/default-node" );
                        i = size;
                    }
                }
            }
            
            return theReturnNode;
            
        }catch( Exception e ){
            throw new ConfigurationLoadException( e.toString() );
        }
        
    }
    
    /**
     * Function loads the configration nodes and stores them with their 
     * information to a map. This map contains a key value pair so that the
     * key is the configration nodes name and the value is the configration
     * nodes object reference.
     *
     * @param doc the XML doc allready loaded to DOM tree
     * @return a map of configuration nodes
     */
    public static JTree loadNodeMap( Document doc ) throws ConfigurationLoadException {
    
        try
        {
            System.err.println("ConfigFileDAO::loadNodeMap()");
            //First we will create the tree
            Element rootElement = doc.getDocumentElement();
            System.err.println("ConfigFileDAO::loadNodeMap()::parseTree");
            //Then we will search the first element whose name is "node".
            
            NodeList childNodes = ((Node)rootElement).getChildNodes();
            int size = childNodes.getLength();
            
            Element theFirstNodeElement = null;
            
            //Now we must found the element named "nodes"
            //This element contains all nodes and their child nodes.
            for( int i = 0; i < size; i++)
            {
                Node theNode = childNodes.item(i);
                if( theNode.getNodeType() == Node.ELEMENT_NODE )
                {
                    Element theElement = (Element)theNode;
                    String elementName = theElement.getTagName();
                    if( elementName.equals("nodes") )
                    {
                        theFirstNodeElement = theElement;
                        i = size;
                    }
                }
            }

            String nodeName = theFirstNodeElement.getTagName();
            
            if( theFirstNodeElement == null )
                throw new ConfigurationLoadException("No configuration node found in the configuration file");
            
            //Then next we must loop all nodes inside the nodes element. Note
            //That the nodes can have child nodes (here we means the elements 
            //whose name is "node". The parsing of these nodes is handled
            //by the parseNodesFromTree method.
            DefaultMutableTreeNode topNode = new DefaultMutableTreeNode("nodes");
            
            NodeList nodeElementList = ((Node)theFirstNodeElement).getChildNodes();
            size =  nodeElementList.getLength();
            
            //Now we will loop all nodes!
            for( int i = 0; i < size; i++)
            {
                Node theNode = nodeElementList.item(i);
                if( theNode.getNodeType() == Node.ELEMENT_NODE )
                {
                    Element theElement = (Element)theNode;
                    String elementName = theElement.getTagName();
                    if( elementName.equals("node") )
                    {
                        DefaultMutableTreeNode aNode = parseNodesFromTree( theElement, "" );
                        topNode.add( aNode );
                    }
                }
            }            
            
            //To last will we create the tree!
            //Note that the tree is a swing tree. This can be used 
            //latter for example for a admin tool.
            JTree tree = new JTree( topNode  );
            return tree;
            
        }catch( Exception e ){
            throw new ConfigurationLoadException( "ConfigFileDAO::LoadNodeMap::exc:: " + e.toString() );
        }
    }
    
   
    /**
     * Loads a configuration node from the information inside the element.
     *
     */
    private static ConfigurationNode loadConfigurationNode( Element theElement, String path ) throws ConfigurationLoadException {
        
        try
        {
            String name = theElement.getAttribute("name");
            String adminGroupName = theElement.getAttribute("admin-group");
            
            HashMap eventMappings = loadEventMappings( theElement );
            HashMap webModels = loadWebModels( theElement );
            HashMap viewMappings = loadViewMappings( theElement );
            
            ConfigurationNode configNode = new ConfigurationNode( name, adminGroupName, eventMappings, webModels, viewMappings, path );
            return configNode;
            
        }catch( Exception e ){
            throw new ConfigurationLoadException( "ConfigFileDAO::loadConfigurationNode::exc:: " + e.toString() );
        }
    }
    
    
    /**
     * Method loads the event mappings part form the configuration files one 
     * node. Note that this fuction loads the configuration only from one 
     * node not from it's descanding nodes. Here we mean with node the element
     * whose name is "node".
     *
     * @param fileStream the strem from the configuration file
     * @return a hasmap that contains EventConfig objects created from the 
     *          configuration file.
     */
    private static HashMap loadEventMappings( Element rootElement ) throws ConfigurationLoadException{
    
        try {
        
            HashMap eventMappingsMap = new HashMap();
            NodeList eventChildList = ((Node)rootElement).getChildNodes();
            int amountOfChilds = eventChildList.getLength();
            
            Element eventMappings = null;
            //Now we will search for element whose name is event-mappings.
            for( int i = 0; i < amountOfChilds; i++ )
            {
                Node theNode = eventChildList.item(i);
                if( theNode.getNodeType() == Node.ELEMENT_NODE )
                {
                    Element theElement = (Element)theNode;
                    if( theElement.getTagName().equals("event-mappings") )
                    {
                        eventMappings = theElement;
                        i = amountOfChilds;
                    }
                }              
            }
            
            //Let's check if we found a event mapping element
            if( eventMappings == null )
            {
                //Nope
                return eventMappingsMap;
            }
            
            //Note that we will only use the first element whose name is 
            //event-mappings. (In the node)
            NodeList eventMappingsNodes = ((Node)eventMappings).getChildNodes();
            int amountOfMappingsChilds = eventMappingsNodes.getLength();
            
            //Next we will loop and search for elements whose name
            //is "event-processor". If it is found a new event mapping
            //configuration object is stored to a has map.
            for( int i = 0; i < amountOfMappingsChilds; i++ )
            {
                Node theNode = eventMappingsNodes.item(i);
                if( theNode.getNodeType() == Node.ELEMENT_NODE )
                {
                    Element theElement = (Element)theNode;
                    if( theElement.getTagName().equals("event-processor") )
                    {
                        //Yep, we found a element whose name 
                        //is event-processor. Let's extract all nececcarily
                        //information from it! First the basic information
                        //like the name, request url and so on..
                        String request = theElement.getAttribute("request");      //The request url
                        String className = theElement.getAttribute("class");      //The event processor class
                        String name = theElement.getAttribute("name");            //The name of the event
                        String view = theElement.getAttribute("view");            //The default template for the processor
                        String level = theElement.getAttribute("authentication"); //Requires authentication?
                        
                        boolean authentication = false;
                        if( level.toLowerCase().equals("true") )
                        {
                            authentication = true;
                        }
                        
                        //Next we will search for the parameters for the event.
                        //These parameters are the function parameters.
                        NodeList eventMethodList = ((Node)theElement).getChildNodes();
                        int eventListSize = eventMethodList.getLength();
                        HashMap eventProcessorMethods = new HashMap();

                        for( int a = 0; a < eventListSize; a++)
                        {
                            Node eventNode = eventMethodList.item(a);
                            if( eventNode.getNodeType() == Node.ELEMENT_NODE )
                            {
                                Element eventMethodElement = (Element)eventNode;
                                if( eventMethodElement.getTagName().equals("event") )
                                {
                                    //Now we will extract the information from
                                    //the event element
                                    String eventName = eventMethodElement.getAttribute("name");
                                    String eventView = eventMethodElement.getAttribute("view");
                                    EventConfig eventConfig = null;
                                    NodeList eventPrametersList = eventMethodElement.getElementsByTagName("param");
                                    System.err.println("param length: "+ eventPrametersList.getLength() );
                                    if( eventPrametersList.getLength() > 0 )
                                    {
                                        int eventParamSize = eventPrametersList.getLength();
                                        LinkedList params = new LinkedList();
                                        for( int b = 0; b < eventParamSize;b++)
                                        {
                                            Element aEvent = (Element)eventPrametersList.item(b);
                                            String paramName = aEvent.getAttribute("name");
                                            String paramType = aEvent.getAttribute("type");

                                            System.err.println("event::param::"+paramName+"::"+paramType);
                                            params.addLast( paramName );
                                        }

                                        eventConfig = new EventConfig( eventName, eventView, params );

                                    }else{
                                        eventConfig = new EventConfig( eventName,eventView, new LinkedList() );
                                    }
                                    eventProcessorMethods.put( eventName, eventConfig );
                                }
                            }
                        }
                        
                        //To last we will create te eventprocessor config, put it to the
                        //has map and loop for a another configuration!
                        EventProcessorConfig eventProcessorConfig = new EventProcessorConfig(
                                                                            request, 
                                                                            className, 
                                                                            name, 
                                                                            view, 
                                                                            eventProcessorMethods, 
                                                                            authentication);

                         eventMappingsMap.put( eventProcessorConfig.getRequest(), eventProcessorConfig );
                    }
                }              
            }
            
            return eventMappingsMap;
            
        }catch( Exception e ) {
            throw new ConfigurationLoadException("loadEventMappings: "+ e.toString() );
        }
    }
    
    
    /**
     * Loads the web model information from one node inside the configuration 
     * file.
     *
     *
     */
    private static HashMap loadWebModels( Element rootElement ) throws ConfigurationLoadException {

        try {
           
            HashMap webModels = new HashMap(); 
            NodeList childList = ((Node)rootElement).getChildNodes();
            int amountOfChilds = childList.getLength();
            
            for( int i = 0; i < amountOfChilds; i++ )
            {
                Node theNode = childList.item(i);
                if( theNode.getNodeType() == Node.ELEMENT_NODE )
                {
                    Element webModelsElement = (Element)theNode;
                    if( webModelsElement.getTagName().equals("web-models") )
                    {
                        i = amountOfChilds;
                        NodeList webModelsList = webModelsElement.getElementsByTagName("model");
                        int size = webModelsList.getLength();
                        for( int a = 0; a < size; a++ ) 
                        {
                            Element modelElement = (Element)webModelsList.item(a);

                            String name = modelElement.getAttribute("name");
                            String scope = modelElement.getAttribute("scope");
                            String className = modelElement.getAttribute("class");

                            ModelConfig modelConfig = new ModelConfig( name, scope, className );
                            webModels.put( name, modelConfig );
                        }
                    }
                }              
            }
          
           return webModels;
           
        }catch( Exception e ) {
            throw new ConfigurationLoadException("loadWebModels: "+ e.toString() );
        }        
    }

    /**
     * Loads the view mappings from configuration node in the configuration
     * file.
     *
     */
    private static HashMap loadViewMappings(  Element rootElement ) throws ConfigurationLoadException{

        try {
            
            HashMap viewMappings = new HashMap();
            
            NodeList childList = ((Node)rootElement).getChildNodes();
            int amountOfChilds = childList.getLength();
            for( int i = 0; i < amountOfChilds; i++ )
            {
                Node theNode = childList.item(i);
                if( theNode.getNodeType() == Node.ELEMENT_NODE )
                {
                    Element viewMappingsElement = (Element)theNode;
                    if( viewMappingsElement.getTagName().equals("view-mappings") )
                    {
                        i = amountOfChilds;
                        NodeList viewList = viewMappingsElement.getElementsByTagName("view");
                        int size = viewList.getLength();
                        for( int a = 0; a < size; a++ ) {

                            Element viewElement = (Element)viewList.item(a);
                            String name = viewElement.getAttribute("name");
                            String template = viewElement.getAttribute("template");
                            System.err.println("working on view::"+name);
                            HashMap pageElements = new HashMap();
                            NodeList pageElementList = viewElement.getElementsByTagName("page-element");
                            int sizePageElements = pageElementList.getLength();
                            
                            for( int b = 0; b < sizePageElements; b++ ) 
                            {
                                Element pageElement = (Element)pageElementList.item(b);
                                String namep = pageElement.getAttribute("name");
                                String value = pageElement.getAttribute("value");
                                PageElementConfig pageElementConfig = new PageElementConfig( name, value );
                                pageElements.put( namep, pageElementConfig );
                            }

                            ViewConfig viewConfig = new ViewConfig( name, template, pageElements );

                            viewMappings.put( name, viewConfig  );
                        }                          
                    }
                }              
            }
            
            return viewMappings;
            
        }catch( Exception e ) {
            throw new ConfigurationLoadException("loadViewMappings:"+ e.toString() );
        }        
    }
}
