/************************************************************************
 * Spooky - a service framework for the Internet
 * EventImplDefault.java
 *
 * Copyright (C) 2001 Erik Karlsson.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the 
 * License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *************************************************************************/
package spooky.control.event;

import javax.servlet.*;
import javax.servlet.http.*;
import java.lang.reflect.*;
import java.util.*;
import spooky.config.*;
import spooky.exception.*;
import spooky.security.*;
import spooky.debug.*;

/**
 *  <p>
 *  An implementation of the event handler interface. This is the default
 *  event implementation used in the framework. Note that the users
 *  can create their own implementations if needed. Note that this 
 *  implementation is intended to use in the web enviroment where
 *  the context contains http request object.
 *  </p>
 *  <ul>
 *      <li>26.7.2001: the first implementation of the class. -EK</li>
 *      <li></li>
 *  </ul>  
 *
 * @author  Erik Karlsson 
 * @version 
 */
public class EventImplDefault implements java.io.Serializable, spooky.control.event.EventHandler{

    private String name;
    private String className;
    private HashMap parameters;
    private String actionName;
    private EventProcessor eventProcessor;
    private EventProcessorConfig eventProcessorConfig;
    private HashMap eventProcessorMap;
    private FrameworkContext securityContext;
    
    /**
     *  Creates an new EventImplDefault. The configuration information is used
     *  to load the right event class. Note that all event classes must extend
     *  in some way the EventProcessor class.
     *
     *  @param name the name of the event
     *  @parameters the parameters of the request
     */
    public EventImplDefault( EventContext eventContext, EventProcessorConfig eventProcessorConfig ) throws EventInitializeException {
        
        this.name = eventProcessorConfig.getName();
        this.className = eventProcessorConfig.getEventClass();
        this.eventProcessorConfig = eventProcessorConfig;
        
        this.eventProcessorMap = new HashMap();
        //To next we will get the event processor...
        try 
        {
            SimpleTrace.printMessage("EventImplDefault::EventImplDefault()::loadClass("+className+")");
            
            /*
            Class theClass = getClass();
            ClassLoader theClassLoader = theClass.getClassLoader();
            Class eventClass = theClassLoader.loadClass( className );
            SimpleTrace.printMessage("EventImplDefault::EventImplDefault()::loadClass::vaihe1");
            Object objProcessor = eventClass.newInstance();
            SimpleTrace.printMessage("EventImplDefault::EventImplDefault()::loadClass::vaihe2");
            eventProcessor = (EventProcessor)objProcessor;
            */
            eventProcessor = (EventProcessor)getClass().getClassLoader().loadClass( className ).newInstance();
            SimpleTrace.printMessage("EventImplDefault::EventImplDefault()::init()");
            eventProcessor.init( eventContext, eventProcessorConfig );
        }
        catch( Exception e )
        {
            throw new EventInitializeException( e.toString() );
        }
        
    }

    public void initEventProcessor() throws EventProcessingException {
        try{
            SimpleTrace.printMessage("EventImplDefault::initEventProcessor()::initEvent()");
            eventProcessor.initEvent();
        }catch( Exception e ) 
        {
           String err_description = e.toString();
           String err_className = "EventImplDefault";
           String err_functionName = "initEventProcessor";
                    
           ErrorObject errorObject = new ErrorObject(err_description, err_className, err_functionName);
           EventProcessingException exp = new EventProcessingException();
           exp.AddErrorObject( errorObject );
           exp.AddClass("EventImplDefault");
           throw exp;               
        }
    }
   
    private void throwErrorException( String err_description, String err_className, String err_functionName) throws EventProcessingException
    {
        ErrorObject errorObject = new ErrorObject(err_description, err_className, err_functionName);
        EventProcessingException exp = new EventProcessingException();
        exp.AddErrorObject( errorObject );
        exp.AddClass("EventImplDefault");        
        throw exp;
    }
    /**
     *  <p>
     *  Method raisen an event that the event processor will process.
     *  The event processor is the class that represents the services
     *  entrypoint.
     *  </p>
     *  <p>
     *  </p>
     *
     *  @param eventContext the current Event Context
     *  @return a EventResponse class from the EventProcessor
     *  @throw  
     */
    public EventResponse raiseEvent(EventContext eventContext ) throws EventProcessingException {
    
        try 
        {
            eventProcessor.init( eventContext, eventProcessorConfig );
            
            //First we will initialize the context of the current
            //request.
            HashMap parameters = eventContext.getParameters();
            String eventName = eventProcessorConfig.getName();
            
            //Let's start parse the request
            //FIrst will we parse the name of the event. Note that if
            //the request doesn't contain a event name we will use
            //the PageLoad event.
            String eventToBeRaised;
            
            if( parameters != null && parameters.containsKey("action") ) 
            {
                eventToBeRaised = (String)parameters.get("action");

                //Now we will check if the event that is intended to be
                //raised is configurated
                HashMap eventMappings = eventProcessorConfig.getEvents();
                if( !eventMappings.containsKey( eventToBeRaised ) )
                {
                    //We will generate an error and throw it
                    String err_description = "Requested Event method "+eventToBeRaised+" was not found in configurations";
                    String err_className = "EventImplDefault";
                    String err_functionName = "raiseEvent";
                    
                    throwErrorException(err_description, err_className, err_functionName);
                }

                EventConfig eventConfig = (EventConfig)eventMappings.get( eventToBeRaised );
                
                //Next we will get all methods present for the current
                //event processor class.
                Class theProcessorClass = ((Object)eventProcessor).getClass();
                Method[] availableMethods = theProcessorClass.getMethods();
                boolean found = false;
                
                Method[] eventMethods = new Method[2];
                int tempA = 0;
                
                //Then we will search for the requested method. If not
                //found we will throw an exception! Note that
                //we will collect all methods that has the right name!
                for( int i = 0; i < availableMethods.length; i++ )
                {
                    if( availableMethods[i].getName().equals( eventToBeRaised ) )
                    {
                        eventMethods[tempA] = availableMethods[i];
                        found = true;
                        //The maximum amount of methods are 2
                        if( tempA ==1  )
                        {
                            tempA++;
                            i = availableMethods.length;
                        }else
                            tempA++;
                    }
                }
                
                //Let's check if we found the method
                if( !found )
                {
                    //We will generate an error and throw it
                    String err_description;
                    if( !found )
                        err_description = "Requested Event method "+eventToBeRaised+" was not found";
                    else
                        err_description ="Null Pointer exception: Requested Event method "+eventToBeRaised+" was not found ";
                   
                    throwErrorException(err_description,"EventImplDefault" , "raiseEvent");
                }
                
                //We found the event function. Let's parse the parameters
                //that it requires. Note that an event function can not
                //be overloaded. First we will load the parameter names 
                //for the function, note that the values are in right order
                //in the hasmap!
                LinkedList eventParams = eventConfig.getEventParams();

                //Jep, all methods must have one method with no parameters
                //and possible other method that can contain parameters
                //Let's found both methods (if there is present more than one)
                //and check which one is the one user wants to access now.
                Method theEventMethod = null;
                Class[] parameterTypes = null;
                int parameterAmount = 0;
                
                int amountOfParameters = parameters.size();
                if( amountOfParameters == 1 )
                {
                    //Jep. User wants to access the method with
                    //no parameters. Note that the request may 
                    //not contain any "useless" parameters too!
                    for( int a = 0; a < 2; a++ )
                    {
                        theEventMethod = eventMethods[a];
                        parameterTypes =  theEventMethod.getParameterTypes();
                        parameterAmount = parameterTypes.length;
                        if( parameterAmount == 0 )
                            a = 2;
                    }
                }
                else
                {
                    //Jep. User wants to access the method with
                    //parameters. 
                    for( int a = 0; a < 2; a++ )
                    {
                        theEventMethod = eventMethods[a];
                        parameterTypes =  theEventMethod.getParameterTypes();
                        parameterAmount = parameterTypes.length;
                        if( parameterAmount > 0 )
                            a = 2;
                    }                    
                }
                

                //Now we will check if there is present all required
                //parameters or if the call is empy. note if
                //the call is empty it indicates that the 
                //user wants to request the empty[methodName] function
                if( eventParams.size() == parameterAmount )
                {
                    Object[] paramList = new Object[parameterAmount];

                    for( int i = 0; i < parameterAmount; i++ )
                    {
                        //Note that if we doesn't found all required parameters
                        //an exception is thrown

                        String paramName = (String)eventParams.get(i);
                        String paramValue = (String)parameters.get( paramName );

                        if( paramValue == null )
                            throw new NoSuchElementException("Parameter value for "+paramName+" doesn't exist");

                        Class methodParameter = parameterTypes[i];

                        if( Boolean.TYPE == methodParameter )
                        {
                            paramList[i] = new Boolean( paramValue );
                        }
                        else if( Byte.TYPE == methodParameter )
                        {
                            paramList[i] = new Byte( paramValue );
                        }
                        else if( Integer.TYPE == methodParameter )
                        {
                            paramList[i] = new Integer( paramValue );
                        }
                        else if( Double.TYPE == methodParameter )
                        {
                            paramList[i] = new Double( paramValue );
                        }
                        else if( Float.TYPE == methodParameter )
                        {
                            paramList[i] = new Float( paramValue );
                        }
                        else if( Long.TYPE == methodParameter )
                        {
                            paramList[i] = new Long( paramValue );
                        }
                        else if( Short.TYPE == methodParameter )
                        {
                            paramList[i] = new Short( paramValue );
                        }
                        else 
                        {
                            paramList[i] =(Object)paramValue ;
                        }                    
                    }
                    
                    //To next we will invoke the method that corresponds to the
                    //event. Note that we will refresh the security context of the
                    //current event.
                    eventProcessor.setSecurityContext( securityContext );    
                    EventResponse eventResponse = (EventResponse)theEventMethod.invoke( eventProcessor , paramList );
                    
                    //Next we will check if the event processing caused errors.
                    if( eventResponse.errors() )
                    {
                        //There was errors. Next we will get all error objects
                        //inside the response object.
                        LinkedList errorObjects = eventResponse.getErrorObjects();
                        EventProcessingException expp = new EventProcessingException();
                        expp.addErrorObjectsList( errorObjects );
                        expp.AddClass( "EventImplDefault" );
                        throw expp;
                    }

                    return eventResponse;
                }
                else if( parameterAmount == 0 ) 
                {
                    //Jep, no parameters!
                    Object[] paramList = new Object[0];
                    
                    //To next we will invoke the method that corresponds to the
                    //event. Note that we will refresh the security context of the
                    //current event.
                    eventProcessor.setSecurityContext( securityContext );    
                    EventResponse eventResponse = (EventResponse)theEventMethod.invoke( eventProcessor , paramList );    
                    
                    //Next we will check if the event processing caused errors.
                    if( eventResponse.errors() )
                    {
                        //There was errors. Next we will get all error objects
                        //inside the response object.
                        LinkedList errorObjects = eventResponse.getErrorObjects();
                        EventProcessingException expp = new EventProcessingException();
                        expp.addErrorObjectsList( errorObjects );
                        expp.AddClass( "EventImplDefault" );
                        throw expp;
                    }

                    return eventResponse;                    
                    
                }
                else
                {
                    //Too many parameters, or the system is not configurated propertly
                    String err_description ="eventfunction parameters mismatch with configurations";
                    throwErrorException(err_description,"EventImplDefault" , "raiseEvent");
                }
            }
            else
            {
                //We will invoke the pageLoadEvent. This event is allways
                //invoked when the "tool" is activated without any specific
                //event. Security context is also refreshed.
                eventProcessor.setSecurityContext( securityContext );
                EventResponse eventResponse = eventProcessor.pageLoadEvent();
                
                //Next we will check if the event processing caused errors.
                if( eventResponse.errors() )
                {
                    //There was errors. Next we will get all error objects
                    //inside the response object.
                    LinkedList errorObjects = eventResponse.getErrorObjects();
                    EventProcessingException expp = new EventProcessingException();
                    expp.addErrorObjectsList( errorObjects );
                    expp.AddClass( "EventImplDefault" );
                    throw expp;
                }

                return eventResponse;
            }
            
        }
        catch( NoSuchElementException  nsee )
        {
           String err_description = nsee.toString();
            throwErrorException(err_description,"EventImplDefault" , "raiseEvent");           
        }
        catch( EventProcessingException epe )
        {
            throw epe;
        }
        catch( Exception e ) 
        {
           String err_description = e.toString();
            throwErrorException(err_description,"EventImplDefault" , "raiseEvent");        
        }
        return null;
        
    }
    
    /**
     *  <p>
     *  Method is called when the event processor has finished up the
     *  event processing. Within this function call the event processor
     *  can make some clean up if needed.
     *  </p>
     */
    public void cleanEvent() 
    {
        eventProcessor.cleanEvent();
    }
    
    /**
     *
     *  @return the name of the event
     */
    public String getName() {
        return this.name;
    }
    
    /**
     * Sets the event handlers security context
     *
     * @param securityContext the security context
     */
    public void setFrameworkContext(FrameworkContext securityContext) {
        this.securityContext = securityContext;
    }
    
}
