/************************************************************************
 * Spooky - a service framework for the Internet
 * Controller.java
 *
 * Copyright (C) 2001 Erik Karlsson.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the 
 * License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *************************************************************************/
package spooky.control.web;           

import javax.servlet.*;
import javax.servlet.http.*;

import java.io.*;
import java.net.*;
import java.util.*;
import spooky.config.*;
import spooky.control.event.*;
import spooky.security.*;
import spooky.exception.*;
import spooky.security.auth.*;
import spooky.security.auth.login.*;
import spooky.model.web.*;
import spooky.view.web.*;

import javax.security.auth.*;
import javax.security.auth.callback.*;
import javax.security.auth.login.*;
import javax.security.auth.spi.*;
/** 
 * <p>
 * The controller class is a Servlet that is the entry point for request
 * to the framework. The Controller servlet activates the request handler
 * and event handlers to process the request that was sent to the controller.
 * </p>
 *
 * <p>
 * The controller determines the request's target from the URL path. For
 * example if the url is [host]/spooky/control/MyAction, the controller
 * will look inside the configurations for MyAction event and map try to
 * map the request to this event. For more information how this mapping is
 * done look at the RequestProcessor class.
 * </p>
 *
 * <p>
 * (future)
 * The controller determines the request's target from the URL path. For
 * example if the url is [host]/spooky/control/client/MyAction, the controller
 * will look inside the configurations for the specific client to seek the 
 * configurations for the MyAction event and map try to map the request to this 
 * event. For more information how this mapping is done look at the RequestProcessor 
 * class.
 * (/future)
 * </p>
 * @author Erik Karlsson
 * @version 0.4
 * @history 22.7.2001 (7/22/2001) Code was commented
 *
 */
public class Controller extends HttpServlet {
   
    private static String CONFIGURATION_FILE = "/WEB-INF/xml/config.xml";
    
    /** <p>
     * Function is called at the start up of the Servlet. Note that the function
     * is called only once when the servlet is started at the first time. The function
     * will load the configuration file for the framework. This configuration is
     * saved to objects for latter access.
     * </p>
     *
     * @param config The servlet Configurations
     * @throws ServletException An exception is throw for example if the 
     *          configuration files where not
     *          available or the loading was unsuccesfully.
     */  
    public void init(ServletConfig config) throws ServletException 
    {
        super.init(config);
        //The configuration file is loaded here. The configurations are 
        //stored to a object created from Configurations class. The object
        //will be stored to the application context (servlet context).
        //Note that the configuration file are only loaded once to the
        //context
        
        ServletContext servletContext = getServletContext();
        
        if( servletContext.getAttribute( Configurations.CONTEXT_NAME ) == null ) {
         
            try {
             
                String configFile = getConfigurationFile();
                String configurationFile = servletContext.getResource( configFile ).toString();
                URL configFileUrl = new URL( configurationFile  );
                Configurations theConfigurations = new Configurations( configFileUrl.openStream() );

                servletContext.setAttribute( Configurations.CONTEXT_NAME, theConfigurations );
                
            }catch( Exception e ) {
                throw new ServletException("Init:: "+e.toString() ); 
            }
            
        }
        
    }

    /**
     *  <p>
     *  Method returns the configuration file's path. 
     *  </p>
     *  @return the configuration file's path
     */
    public String getConfigurationFile(){
        return Controller.CONFIGURATION_FILE;
        //In future this should be loaded from the servlet context
        //[TODO]
    }    
    
    /** 
     * Destroys the servlet.
     */  
    public void destroy() {

    }

    /**
     * Returns the framework's configurations as an Configuration object stored 
     * to the servlet context
     *
     * @return Configurations object
     */
    public Configurations getConfigurations() {
        
        ServletContext servletContext = getServletContext();
        Configurations configurations = (Configurations)servletContext.getAttribute( Configurations.CONTEXT_NAME );
        return configurations;
        
    }

    /**
     * Return a model manager. If the model manager is not allready associated
     * to this session the manager will be created.
     *
     * @return model manager
     */
    public ModelManager getModelManager( ServletContext servletContext, HttpSession httpSession ) 
    {
        ModelManager modelManager = (ModelManager)httpSession.getAttribute( ModelManager.CONTEXT_NAME );
        if( modelManager == null ) 
        {
            modelManager = new ModelManager( servletContext, httpSession );
            httpSession.setAttribute( ModelManager.CONTEXT_NAME, modelManager );
        }
        return modelManager;
    }  
    
    /**
     * Function creates a new Request Processor.
     *
     * @param theSession current HTTP session (for the user)
     * @param servletContext current application context
     * @return a Request Processor from the context, or new one
     */
    public RequestProcessor getRequestProcessor( HttpSession theSession ) {
        
        RequestProcessor requestProcessor = (RequestProcessor)theSession.getAttribute( RequestProcessor.CONTEXT_NAME );
        if( requestProcessor == null ) {
            
	    ServletContext servletContext = getServletContext();
            requestProcessor = new RequestProcessor( theSession, servletContext);
            theSession.setAttribute( RequestProcessor.CONTEXT_NAME, requestProcessor );
            
        }
        
        return requestProcessor;
        
    }

    /**
     * Method returns the current view manager for the application
     *
     * @return a view manager
     */
    public ViewManager getViewManager( HttpSession theSession, HashMap viewMappings ) {
        
        ViewManager viewManager = (ViewManager)theSession.getAttribute( ViewManager.CONTEXT_NAME );
        if( viewManager == null ) {

            viewManager = new ViewManager( viewMappings );
            theSession.setAttribute( ViewManager.CONTEXT_NAME, viewManager );

        }
        
        return viewManager;
    }
    
    /**
     * Returns framework context. Context is specific for certain authenticated
     * user.
     *
     */
    public FrameworkContext getFrameworkContext( HttpServletRequest request, HttpSession theSession  ) 
    {
        FrameworkContext frameworkContext = null;
        //Next we will check if the request contains an authentication
        //session id.
        /*
        Cookie[] theCookies = request.getCookies();
        Cookie authenticationSessionCookie = null;
        if( theCookies != null )
        {
            for( int a = 0; a < theCookies.length; a ++ )
            {
                if( theCookies[a].getName().equals("user_session") )
                {
                    authenticationSessionCookie = theCookies[a];
                    a =theCookies.length; 
                }
            }
        }
        
        //Let's check if the authentication session was found.
        //If it was found let's return it, else the null value.
        if( authenticationSessionCookie != null ) 
        {
         */
            //We must check if the session id matches (else we must try to authenticate the user
            frameworkContext = (FrameworkContext)theSession.getAttribute( FrameworkContext.CONTEXT_NAME );
            if( frameworkContext == null )
            {
                //Unexpected happenened. Perhaps the framework went down and
                //a new frameworkContext is required
                return new FrameworkContext();
            }
        /*    
        }
        else
        {
            return new FrameworkContext();
        }*/

        return frameworkContext;
    }
    
    /**
     * Method stores the security context to the HTTP session and creates a 
     * cookie that is used to identify the authentication session.
     *
     */
    public void storeFrameworkContext( FrameworkContext securityContext, HttpServletResponse response,  HttpSession theSession  )
    {
        //First we will store the security context id
        //to the http cookie
        //SystemPrincipal systemPrincipal = securityContext.getSystemPrincipal();
        
        //Cookie authenticationSessionCookie = new Cookie("user_session", systemPrincipal.getName() );
        //response.addCookie( authenticationSessionCookie );
        
        //Now we will save the context to the session object
        theSession.setAttribute( FrameworkContext.CONTEXT_NAME, securityContext );
    }
    
    /**
     * Method calls the JAAS api to authenticate the user and creates a 
     * framework context that corresponds to the result (note that if 
     * the authentication was unsuccesfully a exception is thrown).
     *
     * @return the framework context
     */
    public FrameworkContext authenticateUser( HttpServletRequest request ) throws LoginException
    {
        FrameworkLoginContext login = new FrameworkLoginContext("Framework", new FrameworkCallbackHandler( request ) );
        login.login();
        
        FrameworkContext frameworkContext = new FrameworkContext( login );

        return frameworkContext;
    }
    
    
    /** 
     *  Method is the entry point for the request that is sent to the Conroller
     *  servlet. Both the GET and POST request are processed by this method. The
     *  method uses the RequestProcessor and ViewManager to perform the requested
     *  operation. Note that these components process the request in more details,
     *  this function only turns the request to data that is understandable for 
     *  the other components.
     *
     * @param request servlet request
     * @param response servlet response
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response) throws ServletException, java.io.IOException {
        
        try 
        {
            System.out.println("Alku");
            //To nex we will set the default screen.  
            String nextView = "EMPTY";
            
            // Create a user's web session. Note that this is not same as the
            // authentication session.
            HttpSession theSession = request.getSession();
            ServletContext theContext = getServletContext();
            String requestedURL = request.getPathInfo();
            
            // With this we ensures that we have the latest requestMappings for use.
            // For example if the request mappings have been reconfigured after the
            // first start of this server the configuration can have been modified.
            Configurations configurations = getConfigurations();

            //Now we will get the configuration node. Note if any
            //specific node is specified in the request the default
            //node is used.
            ConfigurationNode configNode = configurations.searchNode( requestedURL );
            
            //Initialize attibutes
            HashMap eventMappings = null;
            ViewManager viewManager = null;
            ModelManager theModelManager = null;
            String eventName = null;
            
            //Now we will check if we found the node! (With node we mean the
            //unit)
            if( configNode != null )
            {    
                String nodeName = configNode.getNodeName(); 
                eventMappings = configNode.getEventMappings();
                int debugSize = eventMappings.size();

                //Next we will extract the name of the event.
                StringTokenizer strtok = new StringTokenizer( requestedURL, "/" );
                int size = strtok.countTokens();
                
                for(int i = 0; i < size; i++ )
                    eventName = strtok.nextToken();

                if( eventName == null )
                    throw new ServletException("Unable to extract the event info from url");

                //To next will we request the view manager. If this function
                //is called first time the manager will be created.
                viewManager = getViewManager( theSession, configNode.getViewMappings() );
                theModelManager = getModelManager( theContext, theSession );
                theModelManager.setModelMappings( configNode.getModelMappings() );
            }
            else
            {
                //Let's found the default node 
                //so that we can use the normal configurations view mappings
                //to show the 404 error.
                configNode = configurations.searchNode( "/noEvent" );
                viewManager = getViewManager( theSession, configNode.getViewMappings() );
                
                //Let's ensure that no event is found!
                eventName = "";
                configNode = null;
                eventMappings = new HashMap();
            }
            
            // To nex we will check if the requested url exist in current system.
            // If the requested url is not available on the system we will send a 
            // 404 error response to the client.
            if(  (configNode != null)  && eventMappings.containsKey( eventName ) ) 
            {
                //To next will we get the configurations for the current request.
                //the configurations contains for example the name of the event,
                EventProcessorConfig eventProcessorConfig= (EventProcessorConfig)eventMappings.get( eventName );
                FrameworkContext frameworkContext = getFrameworkContext( request, theSession  );
                frameworkContext.setPath( configNode.getPath() );
                
                //To next we will process the request. We must first check if the
                //request processing requires authentication.
                if( eventProcessorConfig.requiresAuthentication() ) 
                {
                    //The request requires an authentication. First must we check
                    //if the user is currently authenticated, if not we must authenticate
                    //her. 
                    try 
                    {
                        if( !frameworkContext.isAuthenticated() )
                        {
                            //Let's authenticate the user
                            frameworkContext = authenticateUser( request );
                            storeFrameworkContext( frameworkContext, response, theSession  );
                        }
                        
                        RequestProcessor theProcessor = getRequestProcessor( theSession );
                        nextView = theProcessor.processRequest( request, frameworkContext, eventProcessorConfig );
                    }
                    catch( LoginException le ) 
                    {
                        //Jep, the authentication was unsuccessfully.
                        //Let's save the old target and make the login ready
                        //for retry the thing...
                        //RequestState requestState = new RequestState( request );
                        ErrorBean errorBean = new ErrorBean();
                        errorBean.setErrorMessage( le.toString() );
                        theSession.setAttribute( ErrorBean.CONTEXT_NAME, errorBean );
                        nextView = "LOGIN";
                    }
                }
                else 
                {
                    // The request doesn't require authentication.
                    // The request will be done wihout authentication, note that
                    // old authentication will not affect from request.
                    try 
                    {
                        RequestProcessor theProcessor = getRequestProcessor(  theSession );
                        nextView = theProcessor.processRequest( request, frameworkContext ,eventProcessorConfig  );
			
                    }catch( RequestException re ) {
                        throw new ServletException( re.toString() );
                    }catch( Exception e ) {
                        throw new ServletException( e.toString() );
                    }
                }

            }else {

                // The system doesn't support the requested url. We should make
                // and error message 404. 
                ErrorBean errorBean = new ErrorBean();
                errorBean.setErrorMessage("requested page: "+requestedURL+" couldn't be found");
                request.setAttribute( ErrorBean.CONTEXT_NAME, errorBean );
                nextView = "ERROR_404";
            }        
            
            // To next we will forward the request to a view that have been
            // selected by the ViewManager (if allthing did go right). Note
            // that this view is an template. In future this will be changed
            // to more generic form.
           
            ServletConfig servletConfig = getServletConfig();
            ServletContext configContext = servletConfig.getServletContext();
            RequestDispatcher dispatcher = configContext.getRequestDispatcher( viewManager.getTemplate( nextView ) );
            dispatcher.forward( request, response );

        }catch( Exception e ) {
            //throw new ServletException( e.toString() );
            PrintWriter out = response.getWriter();
            if( (Object)e instanceof BaseException )
            {
                Collection errorMessages = ((BaseException)e).getErrorMessages();
                Iterator itr = errorMessages.iterator();
                
                out.println("<html>\n<head>Spooky error</head>\n<body bgcolor=\"#FFFFFF\">\n");
                
                while( itr.hasNext() )
                {
                    out.println("<table>\n<tr>");
                    ErrorObject error = (ErrorObject)itr.next();
                    out.println("<td>Name of the function</td>");
                    out.println("<td>"+error.getFunctionName()+"</td>");
                    out.println("</tr>");
                    
                    out.println("<tr>");
                    out.println("<td>Name of the class</td>");
                    out.println("<td>"+error.getClassName()+"</td>");
                    out.println("</tr>");
                    out.println("</table>");
                    out.println("<p>"+ error.getDescription()+"</p>");
                }
                
            }
            else
            {
                System.err.println("Controller::error::" + e.toString() );
                
                out.println("<html><head><title>Spooky error</title></head><body bgcolor=\"#FFFFFF\">");
                out.println("<h2 class=\"font-family: Arial, Helvetica\">Exception occured</h2>"+ e.toString() );
                out.println("</body></html>");
            }
        }
      
    } 

    /** Handles the HTTP <code>GET</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    } 

    /** Handles the HTTP <code>POST</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, java.io.IOException {
        processRequest(request, response);
    }

    /** Returns a short description of the servlet.
    */
    public String getServletInfo() {
        return "Short description";
    }

}
