/************************************************************************
 * Spooky - a service framework for the Internet
 * RequestProcessor.java
 *
 * Copyright (C) 2001 Erik Karlsson.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the 
 * License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *************************************************************************/

package spooky.control.web;

import javax.servlet.http.*;
import javax.servlet.ServletContext;
import spooky.control.event.*;
import spooky.config.*;
import spooky.model.web.*;
import spooky.security.*;
import spooky.debug.*;


import java.util.*;

/**
 *
 * @author  Erik Karlsson
 * @version 
 */
public class RequestProcessor implements java.io.Serializable {
    
    private HashMap eventProcessors;
    private HashMap events;
    public static String CONTEXT_NAME = "request-processor";   
    
    private ServletContext servletContext;
    private HttpSession httpSession;
    
    /** Creates new RequestProcessor */
    public RequestProcessor( HttpSession httpSession, ServletContext servletContext ) 
    {
        this.events = new HashMap();
        this.httpSession = httpSession;
        this.servletContext = servletContext;
    }


    public ModelManager getModelManager( ServletContext servletContext, HttpSession httpSession ) 
    {
        ModelManager modelManager = (ModelManager)httpSession.getAttribute( ModelManager.CONTEXT_NAME );
        if( modelManager == null ) 
        {
            modelManager = new ModelManager( servletContext, httpSession );
            httpSession.setAttribute( ModelManager.CONTEXT_NAME, modelManager );
        }
        return modelManager;
    }
    
    /** 
     * Returns the framework's configurations as an Configuration object stored 
     * to the servlet context
     *
     * @return Configurations object
     */
    public Configurations getConfigurations() {
        
        Configurations configurations = (Configurations)servletContext.getAttribute( Configurations.CONTEXT_NAME );
        return configurations;
        
    }
  
    /**
     * Method process the event. The even processing is not done here. It is 
     * made in the event processor class. Here we should note that the class
     * can be also an wrapper for an EJB component.
     *
     *  @param request the http request
     *  @param httpSession current user's session
     */
    public String processRequest( HttpServletRequest request, FrameworkContext securityContext, EventProcessorConfig eventProcessorConfig ) throws RequestException 
    {
        try 
        {
             //To next we will create the event context,
            //that contains in this situation the httpsession and request
            //objects
            EventContext eventContext = new WebEnviroment( httpSession, servletContext, request );
            
            //Now we will search the event handler class that will
            //activate the event on the event processor class. Note that
            //the event processor class is selected inside the event
            //handler class. (Normally this handler class is the EventImplDefault)
            EventHandler theEventHandler = null;
            String eventName = eventProcessorConfig.getName();
            
            if( events.containsKey( eventName ) ) 
            {
                theEventHandler = (EventHandler)events.get( eventName );
            }
            else 
            {
                theEventHandler = new EventImplDefault( eventContext, eventProcessorConfig );
                theEventHandler.initEventProcessor();
                events.put( eventName, theEventHandler );
            }
            
            //Now we will update the security context for the event handler. The
            //handler will update the event processor's security context
            //(Framework contentext).
            theEventHandler.setFrameworkContext( securityContext );

            //Then will we activate the handler and take the event response from it!
            EventResponse eventResponse = null;
            eventResponse = theEventHandler.raiseEvent( eventContext );

            if( eventResponse == null )
                throw new RequestException( "RequestProcessor::Event response null" );
            
            //To next we will notify the model manager about the modified models.
            //The model manager will update the web parts of the updated models.
            //(note that we updated the "real" model in the processEvent
            //method.
            if( eventResponse.getUpdatedModels() != null || ((Collection)eventResponse.getUpdatedModels()).size() > 0  )
            {
                ModelManager modelManager = getModelManager( servletContext, httpSession );
                modelManager.notify( eventResponse );                    
            }
                
            if( eventResponse.getNextTemplate().equals("") )
            {
               //let's use the default view of the event processor
               //As you can see this is used if the view is defined as ""
               return eventProcessorConfig.getView();
            }
            else{
                //The service is defined the view. Usually this 
                //is selected by calling the eventConfig.getView() method
                //or last should have... ;)
                return eventResponse.getNextTemplate();
            }
           
        }catch( EventInitializeException eie ) {
            throw new RequestException( eie.toString() );
        }catch( EventProcessingException epe ) {
            throw new RequestException( epe.toString() );
        }catch( Exception e ) {
            throw new RequestException( e.toString() );
        }
    }
    
}
