/************************************************************************
 * Spooky - a service framework for the Internet
 * ModelManager.java
 *
 * Copyright (C) 2001 Erik Karlsson.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the 
 * License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *************************************************************************/
package spooky.model.web;

import java.util.*;

import javax.servlet.http.*;
import javax.servlet.*;

import spooky.config.*;
import spooky.control.event.*;
import spooky.debug.*;
/**
 *
 * @author  Erik Karlsson
 * @version 
 */
public class ModelManager implements java.io.Serializable{

    
    public static String CONTEXT_NAME = "model_manager";
    
    private ServletContext servletContext;
    private HttpSession httpSession;
    private HashMap modelMappings;
    
    /** Creates new ModelManager */
    public ModelManager( ServletContext servletContext, HttpSession httpSession ) {
        
        this.servletContext = servletContext;
        this.httpSession = httpSession;
    }

    /**
     * Returns the model mappings
     *
     * @return the model mappings
     */
    public HashMap getModelMappings() {
        return modelMappings;
    }    
  
    public ModelCollection getSessionModels() {
        
        ModelCollection theCollection = (ModelCollection)httpSession.getAttribute( (String)(ModelCollection.CONTEXT_NAME+"-session") );
        if( theCollection == null ) {
         
            theCollection = new ModelCollection();
            httpSession.setAttribute( (String)(ModelCollection.CONTEXT_NAME+"-session"), theCollection );
        }
        
        return theCollection;
    }
    
    public ModelCollection getContextModels() {
        
        ModelCollection theCollection = (ModelCollection)httpSession.getAttribute( (String)(ModelCollection.CONTEXT_NAME+"-application") );
        if( theCollection == null ) {
         
            theCollection = new ModelCollection();
            httpSession.setAttribute( (String)(ModelCollection.CONTEXT_NAME+"-application"), theCollection );
        }
        
        return theCollection;
    }    

    /*
    public ModelCollection getRequestModels(){
        ModelCollection theCollection = (ModelCollection)httpSession.getAttribute( (String)(ModelCollection.CONTEXT_NAME+"-application") );
        if( theCollection == null ) {
         
            theCollection = new ModelCollection();
            httpSession.setAttribute( (String)(ModelCollection.CONTEXT_NAME+"-application"), theCollection );
        }
        
        return theCollection;        
    }
    */
    
    public void setModelMappings( HashMap modelMappings )
    {
        this.modelMappings = modelMappings;
    }
    
    /**
     *  Returns specific model within the scope. If no model
     *  is found it will be created. If the creation fails
     *  an null value is returned.
     *
     *  @param nameOfTheModel The name of the model
     *  @param scope    scope: <code>session</code> or <code>application</code>
     */
    public Object getModel( String nameOfTheModel, String scope ) {
        
        try {
            SimpleTrace.printMessage("ModelManager::getModel()");
            HashMap modelConfigurations = getModelMappings();
            ModelConfig modelConfig = (ModelConfig)modelConfigurations.get( nameOfTheModel );
            ModelCollection theCollection = null;
            Object obj = null;
            
            SimpleTrace.printMessage("ModelManager::getModel()::check the scope:"+scope);
            if( scope.equals("session") )
            {
                theCollection = getSessionModels();
            }
            else if( scope.equals("application") )
            {
                theCollection = getContextModels();
            }
            
            if( theCollection.containsKey( nameOfTheModel ) )
            {
                obj = theCollection.getModelAsObject( nameOfTheModel );
            }
            else
            {
                WebModel theModel = (WebModel)getClass().getClassLoader().loadClass(  modelConfig.getModelClass() ).newInstance();
                theModel.update( null );
                theCollection.addModel( nameOfTheModel, theModel );
                obj = (Object)theModel;
            }
            return obj;
            
        }catch( Exception e ) {
            System.err.println("ModelManager::getModel::"+e.toString());
        }
        return null;
    }
    
    /**
     * Returns the model from certain context. The context means Session-,
     * request-, or application-context.
     *
     */
    private WebModel getWebModel( String nameOfTheModel, ModelCollection theCollection, ModelConfig modelConfig ) throws ClassNotFoundException {

        try {
            
            if( !theCollection.containsKey( nameOfTheModel ) ) 
            {
                HttpModel theModel = (HttpModel)getClass().getClassLoader().loadClass( modelConfig.getModelClass() ).newInstance();
                theModel.setObjects( httpSession, servletContext );
                theCollection.addModel( nameOfTheModel, ((WebModel)theModel) );
            }
            
            WebModel webModel =theCollection.getModel( nameOfTheModel );
            return webModel;
            
        }catch( ClassNotFoundException e ) {
            throw e;
        }catch( Exception e ) {
            throw new ClassNotFoundException("unspecified");
        }
    }

    /**
     *  This method is called by the request processer. The parameters
     *  contains information of modified models. This is importan!
     *
     */
    public void notify( EventResponse eventResponse ) throws WebModelException {
    
        try {
            SimpleTrace.printMessage("ModelManager::Notify()");
            //First we load the models allready stored to the session and to the
            //servlet context. Note that these containers can also be empty if 
            //no model has not been stored before.
            ModelCollection sessionModels = getSessionModels();
            ModelCollection contextModels = getContextModels();

            //Next we will load the configurations for the web models.
            HashMap modelConfigurations = getModelMappings();
            Iterator modifiedModelsIterator = eventResponse.getUpdatedModels().iterator();

            //Next we will iterate throught the modified models
            while( modifiedModelsIterator.hasNext() ) {
                
                //The collection contains the name of the modified models.
                String nameOfTheModel = (String)modifiedModelsIterator.next();
                SimpleTrace.printMessage("ModelManager::Notify()::Modified model::"+nameOfTheModel);
                //The configurations can be used to map the name with the models
                //classes
                if( modelConfigurations.containsKey( nameOfTheModel ) ) {

                    //Here we load the configurations for the current model.
                    ModelConfig modelConfig = (ModelConfig)modelConfigurations.get( nameOfTheModel );
                    
                    //Let's check the scope. This affect where model will be 
                    //stored if the model doesn't exist allready.
                    String scope = modelConfig.getScope();

                    //Note that if the model doesn't exist allready the 
                    //getWebModel method will also create one and store
                    //it to the right context (session or application).
                    if( scope.equals("session") ) {

                        WebModel webModel = getWebModel( nameOfTheModel, sessionModels, modelConfig );
                        
                        //The update function will update the web part of the model.
                        //note that the web model can get one argument when the model
                        //is updated.
                        webModel.update( eventResponse.getAttribute( nameOfTheModel ) );

                    }else if( scope.equals("application") ) {

                        WebModel webModel = getWebModel( nameOfTheModel, contextModels, modelConfig );
                        
                        //The update function will update the web part of the model.
                        //note that the web model can get one argument when the model
                        //is updated.
                        webModel.update( eventResponse.getAttribute( nameOfTheModel ) );

                    }else if( scope.equals("request") ) {
                        WebModel webModel = getWebModel( nameOfTheModel, contextModels, modelConfig );
                        webModel.update( eventResponse.getAttribute( nameOfTheModel ) );
                    }else {
                        
                        //The configuration file is incorrect. Perhaps a miss spelled
                        //word caused this error.
                        throw new WebModelException("Model "+nameOfTheModel+" scope is wrong, check the spelling please.");
                    }


                }else {
                    throw new WebModelException("The requested model doesn't exist");
                }
            }

        }catch( WebModelException wme )  {
            throw wme;
        }catch( ClassNotFoundException cnfe ) {
            throw new WebModelException("class was not found: "+cnfe.getMessage() );
        }catch( Exception e ) {
            throw new WebModelException( e.toString() );
        }
    }
    
}
