/************************************************************************
 * Spooky - a service framework for the Internet
 * FrameworkContext.java
 *
 * Copyright (C) 2001 Erik Karlsson.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the 
 * License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *************************************************************************/
package spooky.security;

import javax.security.auth.*;
import javax.security.auth.callback.*;
import javax.security.auth.login.*;
import javax.security.auth.spi.*;
import spooky.security.auth.login.*;

import java.util.*;
import spooky.security.auth.*;
/**
 *
 * @author  ekarlsso
 * @version 
 */
public class FrameworkContext {

    private Subject thisSubject;
    private boolean authenticated;
    private String currentPath;
    
    public static String CONTEXT_NAME = "FRAMEWORK_SECURITY_CONTEXT";
    
    /** Creates new FrameworkContext */
    public FrameworkContext( FrameworkLoginContext userLoginContext ) 
    {
        authenticated = true;
        this.thisSubject = userLoginContext.getSubject();
    }

    public FrameworkContext(){
        this.authenticated = false;
        this.thisSubject= null;
    }
    
    public void setPath( String path ){
        this.currentPath = path;
    }
    
    public String getPath(){
        return this.currentPath;
    }
    
    /**
     *  Marks the user's context to be authenticated
     */
    public void setAuthenticated(){
        authenticated = true;
    }
    
    /**
     *  Marks the user's context to be unauthenticated
     */
    public void unsetAuthenticated(){
        authenticated = false;
    }
    
    public boolean isAuthenticated(){
        return authenticated;
    }
    
    /**
     * return the subject if the user is authenticated else the method
     * will return null
     *
     * @return the subject
     */
    public Subject getSubject(){
        
        if( authenticated )
            return thisSubject;
        else
            return null;
    }

    /**
     * Checks if the user is included in the user group. Note that false is
     * also returned if the current user has not been authenticated.
     *
     * @param groupPrincipal the user group
     */
    public boolean isInGroup( FrameworkGroupPrincipal groupPrincipal )
    {
        if( !authenticated )
            return false;
        
        Set groups = thisSubject.getPrincipals();
        int size = groups.size();
        
        Iterator itr = groups.iterator();
        while( itr.hasNext() )
        {
            Object obj = itr.next();
            if( obj instanceof FrameworkGroupPrincipal )
            {
                FrameworkGroupPrincipal group = (FrameworkGroupPrincipal)obj;
                if( group.equals( groupPrincipal ) )
                    return true;
            }
        }   
        
        return false;
    }
  
    /**
     * Returns the system principal for the current framework security
     * context
     *
     * @return system principal
     */
    public SystemPrincipal getSystemPrincipal()
    {
        Set credential = thisSubject.getPrincipals();
        SystemPrincipal systemPrincipal = null;
        
        int size = credential.size();
        
        Iterator itr = credential.iterator();
        boolean loop = itr.hasNext();
        while( loop )
        {
            Object obj = itr.next();
            if( obj instanceof SystemPrincipal )
            {
                systemPrincipal = (SystemPrincipal)obj;
                loop = false;
            }
            else
                loop = itr.hasNext();
        }
        
        return systemPrincipal;
    }
    
    /**
     * Checks if the contexts user is the user that is checked. Note that 
     * false is also returned if the current user has not been authenticated.
     *
     * @param userPrincipal the principal representing the user
     */
    public boolean isUser( FrameworkUserPrincipal userPrincipal )
    {
        if( !authenticated )
            return false;
        
        Set groups = thisSubject.getPrincipals();
        int size = groups.size();
        
        Iterator itr = groups.iterator();
        while( itr.hasNext() )
        {
            Object obj = itr.next();
            if( obj instanceof FrameworkUserPrincipal )
            {
                FrameworkUserPrincipal user = (FrameworkUserPrincipal)obj;
                if( user.equals( userPrincipal ) )
                    return true;
            }
        }   
        
        return false;
    }
    
    /**
     * Returns the name of the user. If the user is not authenticated
     * function will return anonymous as the name of the user.
     *
     * @return the user name
     */
    public String getUserId() 
    {
        if( !authenticated )
            return "anonymous";
        
        Set users = thisSubject.getPrincipals();
        Iterator itr = users.iterator();
        
        while( itr.hasNext() )
        {
            Object obj = itr.next();
            if( obj instanceof FrameworkUserPrincipal )
            {
                FrameworkUserPrincipal user = (FrameworkUserPrincipal)obj;
                user.getName();
            }            
        }
        
        return "anonymous";
    }
}
