/************************************************************************
 * Spooky - a service framework for the Internet
 * UserAdminProcessor.java
 *
 * Copyright (C) 2001 Erik Karlsson.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the 
 * License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *************************************************************************/

package spooky.sysservices.user;

import spooky.config.*;
import spooky.control.event.*;
import spooky.security.*;
import spooky.security.auth.*;
import spooky.users.ejb.*;

import java.util.*;
import javax.ejb.*;
import javax.rmi.*;
import javax.naming.*;
import javax.servlet.http.*;
/**
 * <p>
 * Class is one othe admin event processors. This processors contains
 * the logic to manage users and user groups. The task that this processors
 * is capable to do is
 * </p> 
 * <ul>
 *  <li>Add Users</li>
 *  <li>Remove users</li>
 *  <li>Add User Groups</li>
 *  <li>Remove User Groups</li>
 *  <li>Add users to certain group</li>
 *  <li>Remove users from certain group</li>
 * </ul>
 *
 * @author  Erik Karlsson
 * @version 
 */
public class UserAdminProcessor extends HttpEventProcessor{

    /** Creates new UserAdminProcessor */
    public UserAdminProcessor() {
    }

    
    public EventResponse addUser()
    {
        EventResponse eventResponse = new EventResponse( "ADMIN_ADDUSER" );
        return eventResponse;
    }
    
    /**
     * Event Method adds a new user to the framework. Note that the
     * user can use other users also that are not added in this way.
     * This is done by 
     *
     */
    public EventResponse addUser( String userName, String userPasswd )
    {
        try{
        
            userName.trim();
            userPasswd.trim();
            //First we will check that the current user is
            //authenticated user!
            FrameworkContext securityContext = getSecurityContext();
            EventProcessorConfig eventConfig = getEventProcessorConfig();
            
            if( !securityContext.isAuthenticated() )
            {
                //User is not authenticated!
                //Let's return a error!
                return createErrorResponse( "User not authenticated", "UserAdminProcessor", "addUser" );
            }
            
            //Ok, user is authenticated. Let's create the new user!
            //Let's check first if the user (using this event) is in the
            //built in administrator group
            FrameworkGroupPrincipal adminGroup = getDefaultAdminGroup();
            boolean isAdmin = securityContext.isInGroup( adminGroup );
 
            //Let's check if the user belongs to the default
            //administrator group
            if( isAdmin )
            {
                //Yep, the user belongs to the default admin
                //group. Lets create the user.
                InitialContext cx = new InitialContext();
                Object homeObj = cx.lookup("UserFacadeEJB");
                UserFacadeHome theHome = (UserFacadeHome)PortableRemoteObject.narrow( homeObj, UserFacadeHome.class );
                UserFacade userFacade = theHome.create();

                LinkedList list = new LinkedList();
                HashMap map = new HashMap();
                list.add("admin_message_object");
                
                if( !userFacade.existUser( userName ) ) 
                {
                    userFacade.createUser( userName, "", userPasswd );
                }
                else
                {
                    //Lets create an message that the user was not created!
                    putMessageToModel("The user exist allready. A user was not created. Check the "+
                                "user name and try again.", AdminMessageObject.WARNING);
                 
                    EventConfig theEventConfig = getEventConfig("addUser");
                    EventResponse eventResponse = new EventResponse(list, map, theEventConfig.getView() );
                    return eventResponse;
                }

                //User was created. Let's create a response about it!
                list.add("user_list_model");
                

                //Let's create a response that the user was created
                putMessageToModel( "The user <b>"+userName+"</b> was created succesfully.", AdminMessageObject.OK);
                EventConfig theEventConfig = getEventConfig("addUser");
                EventResponse eventResponse = new EventResponse(list, map, theEventConfig.getView() );
                return eventResponse;
            }
            else
            {
                //The user was not an Admin, so we must show the error page
                //for the user.
                LinkedList list = new LinkedList();
                list.add("admin_message_object");
                HashMap map = new HashMap();
                
                //Let's save the message to the http session, so that 
                //the model can load it from there
                putMessageToModel( "The user is not admin user. The operation was not permited."+
                            "Use user that is admin in the default admin group", AdminMessageObject.ERROR);
                
                EventResponse eventResponse = new EventResponse( list, map, "ADMIN_MESSAGE" );
                return eventResponse;
            }
        
        }catch( Exception e ) {
            return createErrorResponse( e.toString(), ((Object)this).getClass().getName(), "addUser" );
        }
    }

    /**
     * <p>
     * Event removes an user from the Spooky's user database. Note that
     * if the Spooky is configured to use another user data base, this
     * function can not be used against these users.
     * </p>
     * <p>
     * User must be in default admin group to use this method.
     * </p>
     *
     * @param userName the name of the user that will be removed
     */
    public EventResponse removeUser( String userName ) {
       
        try
        {
            userName.trim();

            //First we will check that the current user is
            //authenticated user!
            FrameworkContext securityContext = getSecurityContext();
            EventProcessorConfig eventConfig = getEventProcessorConfig();
            
            if( !securityContext.isAuthenticated() )
            {
                //User is not authenticated!
                //Let's return a error!
                return createErrorResponse( "User not authenticated", "UserAdminProcessor", "addUser" );
            }
            
            //Now we will check that the user is in 
            //default admin group. Only users in the
            //default admin group is allowed to add
            //new users.
            FrameworkGroupPrincipal adminGroup = getDefaultAdminGroup();
            boolean isAdmin = securityContext.isInGroup( adminGroup );
 
            //Let's check if the user belongs to the default
            //administrator group
            if( isAdmin )
            {
                LinkedList list = new LinkedList();
                HashMap map = new HashMap();
                
                //Yep, the user belongs to the default admin
                //group. Lets create the user.
                InitialContext cx = new InitialContext();
                Object homeObj = cx.lookup("UserFacadeEJB");
                UserFacadeHome theHome = (UserFacadeHome)PortableRemoteObject.narrow( homeObj, UserFacadeHome.class );
                UserFacade userFacade = theHome.create();
                
                if( userFacade.existUser( userName ) ) 
                {
                    userFacade.removeUser( userName );
                    
                    //Ok, we modifies the user list and the user
                    //message object!
                    list.add("admin_message_object");
                    list.add("user_list_model");
                    putMessageToModel( "The user <b>"+userName+"</b> was removed succesfully.", AdminMessageObject.OK );
                }
                else
                {
                    //The user doesn't exist
                    list.add("admin_message_object");
                    putMessageToModel( "The user <b>"+userName+"</b> was not found in the user database.", AdminMessageObject.WARNING );                  
                }

                
                HashMap events = eventConfig.getEvents();
                EventConfig theEventConfig = (EventConfig)events.get("addUser");
                
                EventResponse eventResponse = new EventResponse(list, map, eventConfig.getView() );
                return eventResponse;
            }
            else
            {
                //The user was not an Admin, so we must show the error page
                //for the user.
                LinkedList list = new LinkedList();
                list.add("admin_message_object");
                HashMap map = new HashMap();
                
                putMessageToModel("Current user is not in default admin group. "+
                                  "Only users in this user group is allowed to "+
                                  "add new users.", AdminMessageObject.ERROR );
                
                //Then we will notify the framework about the changed model.
                HashMap events = eventConfig.getEvents();
                EventConfig theEventConfig = (EventConfig)events.get("addUser");
                
                EventResponse eventResponse = new EventResponse( list, map, theEventConfig.getView() );
                return eventResponse;
            }
        
        }catch( Exception e ) {
            return createErrorResponse( e.toString(), ((Object)this).getClass().getName(), "addUser" );
        }        
    }  
    
    
    public EventResponse createUserGroup() {
        
        EventResponse eventResponse = new EventResponse( "ADMIN_CREATE_GROUP" );
        return eventResponse;
    }
    
    /**
     * Method creates an new user group. The user group will be administrated
     * by the node's admin group which is defined by the <code>path</code>
     * attribute.
     * 
     * @param groupName the name of the group
     * @param path the node path
     */
    public EventResponse createUserGroup( String groupName, String path )
    {
        try 
        {
            groupName.trim();
            path.trim();
            //First we will check that the current user is
            //authenticated user!
            FrameworkContext securityContext = getSecurityContext();
            EventProcessorConfig eventConfig = getEventProcessorConfig();
            
            if( !securityContext.isAuthenticated() )
            {
                //User is not authenticated!
                //Let's return a error!
                return createErrorResponse( "User not authenticated", "UserAdminProcessor", "addUser" );
            }
            
            boolean allowed = false;

            //First we will check if the user is in default admin group.
            //If the user is in the group the event is automatically
            //allowed.
            FrameworkGroupPrincipal adminGroup = getDefaultAdminGroup();
            if( securityContext.isInGroup( adminGroup ) ){
                allowed = true;
            }else{
            
                //The user was not in the default admin group.
                //Now we must check the user groups path and the
                //try to figure is the user as admin in some of 
                //the nodes. We will iterate all admin groups node by
                //node and check if we found the user.
                StringTokenizer groupPathToken = new StringTokenizer( path, "/" );
                String workPath = "/";
                
                for( int i = 0; i < groupPathToken.countTokens(); i++ )
                {
                    workPath += groupPathToken.nextToken();
                    FrameworkGroupPrincipal nodeAdminGroup = getAdminGroup( workPath );
                    if( nodeAdminGroup == null )
                        return createErrorResponse( "The node "+workPath+" was not configurated propertly", ((Object)this).getClass().getName(), "addUserToGroup" );
                    
                    if( securityContext.isInGroup( nodeAdminGroup ) )
                    {
                        //Jep user is in the group. It is no 
                        //idea to check the rest of the node's 
                        //admin groups (they are childs for the current node)
                        i = groupPathToken.countTokens();
                        allowed = true;
                    }
                }
                
                //Note that if the user is not in the default admin group
                //and also not in any of the nodes found in the path
                //the user is not allowed to admin the system.
            }            
            
            if( allowed )
            {
                LinkedList list = new LinkedList();
                HashMap map = new HashMap();
                list.add("admin_message_object");
                
                InitialContext cx = new InitialContext();
                Object bound = cx.lookup("UserGroupEJB");
                UserGroupHome groupHome = (UserGroupHome)PortableRemoteObject.narrow( bound, UserGroupHome.class );
                Collection groups = groupHome.findByGroupName( groupName );
                
                if( groups.size() != 0 )
                {
                    //Let's save the message to the http session, so that 
                    //the model can load it from there
                    putMessageToModel("The user group <b>"+groupName+"</b> allready exist.", AdminMessageObject.WARNING);

                    //Then we will notify the framework about the changed model.
                    EventConfig theEventConfig = getEventConfig("createUserGroup");
                    EventResponse eventResponse = new EventResponse( list, map, theEventConfig.getView() );
                    return eventResponse;                       
                }
                
                //Ok, we didin't found the group! lets create one!
                UserGroup userGroup = groupHome.create( groupName, path, "");

                //The user is not allowed to create the user group under the
                //defined configuration node (unit)
                list.add("group_list_model");
                
                //Let's save the message to the http session, so that 
                //the model can load it from there
                putMessageToModel("The user group <b>"+
                                    groupName+"</b> was created and is administrated "+
                                    "by the <b>" +path + "</b> unit.", AdminMessageObject.ERROR);

                //Then we will notify the framework about the changed model.
                EventConfig theEventConfig = getEventConfig("createUserGroup");
                EventResponse eventResponse = new EventResponse( list, map,theEventConfig.getView() );
                return eventResponse;                 
            }
            else 
            {
                //The user is not allowed to create the user group under the
                //defined configuration node (unit)
                LinkedList list = new LinkedList();
                HashMap map = new HashMap();
                
                list.add("admin_message_object");
                
                //Let's save the message to the http session, so that 
                //the model can load it from there
                putMessageToModel("The current "+
                            "user is not allowed to create the <b>" +groupName +
                            "</b> user group", AdminMessageObject.ERROR);
                
                //Then we will notify the framework about the changed model.
                EventConfig theEventConfig = getEventConfig("createUserGroup");
                EventResponse eventResponse = new EventResponse( list, map, theEventConfig.getView() );
                return eventResponse;               
                
            }
            
        }catch( Exception e ) {
            return createErrorResponse( e.toString(), ((Object)this).getClass().getName(), "createUserGroup" );
        }
    }    
    
    /**
     * Adds a user to user group. Anyone who is in administrator group
     * can add user to group that is managed by their node, or is administrated 
     * by child node. To use this event the user doesn't require to be in
     * the default admin group.
     *
     * @param userName the user to be added
     * @param groupName the group where the user is added
     */
    public EventResponse addUserToGroup( String userName, String groupName )
    {
        try 
        {
            userName.trim();
            groupName.trim();
            //First we will check that the current user is
            //authenticated user!
            FrameworkContext securityContext = getSecurityContext();
            EventProcessorConfig eventConfig = getEventProcessorConfig();
            
            if( !securityContext.isAuthenticated() )
            {
                //User is not authenticated!
                //Let's return a error!
                return createErrorResponse( "User not authenticated", "UserAdminProcessor", "addUser" );
            }
            
            //First we will check if the user is authenticated and is in admin group
            //In future we will check if the user is in the "path" admin group. Let's check
            //it!
            InitialContext cx = new InitialContext();
            Object bound = cx.lookup("UserGroupEJB");
            UserGroupHome groupHome = (UserGroupHome)PortableRemoteObject.narrow( bound, UserGroupHome.class );
            Collection groups = groupHome.findByGroupName( groupName );
            
            UserGroup userGroup = null;
            
            if( groups.size() == 1 )
            {
                Iterator itr = groups.iterator();
                userGroup = (UserGroup)itr.next();
            }
            else{
                //No group was found or there
                //was too many groups (requires configuration!)
                //Let's create a message for the user
                LinkedList list = new LinkedList();
                list.add("admin_message_object");
                HashMap map = new HashMap();
                
                //Let's save the message to the http session, so that 
                //the model can load it from there
                AdminMessageObject admObj = new AdminMessageObject("The user group "+groupName+" was not found");
                HttpSession httpSession = getHttpSession();
                httpSession.setAttribute( AdminMessageObject.CONTEXT_NAME, admObj );
                
                //Then we will notify the framework about the changed model.
                EventResponse eventResponse = new EventResponse( list, map,eventConfig.getView() );
                return eventResponse;                
            }
            
            //Ok! we found the user group. Now we will check if the user
            //has rights to add the user to the group!
            String groupPath = userGroup.getGroupPath();
            boolean allowed = false;

            //First we will check if the user is in default admin group.
            //If the user is in the group the event is automatically
            //allowed.
            FrameworkGroupPrincipal adminGroup = getDefaultAdminGroup();
            if( securityContext.isInGroup( adminGroup ) ){
                allowed = true;
            }else{
            
                //The user was not in the default admin group.
                //Now we must check the user groups path and the
                //try to figure is the user as admin in some of 
                //the nodes. We will iterate all admin groups node by
                //node and check if we found the user.
                StringTokenizer groupPathToken = new StringTokenizer( groupPath, "/" );
                String workPath = "/";
                
                for( int i = 0; i < groupPathToken.countTokens(); i++ )
                {
                    workPath += groupPathToken.nextToken();
                    FrameworkGroupPrincipal nodeAdminGroup = getAdminGroup( workPath );
                    if( nodeAdminGroup == null )
                        return createErrorResponse( "The node "+groupPath+" was not configurated propertly", ((Object)this).getClass().getName(), "addUserToGroup" );
                    
                    if( securityContext.isInGroup( nodeAdminGroup ) )
                    {
                        //Jep user is in the group. It is no 
                        //idea to check the rest of the node's 
                        //admin groups (they are childs for the current node)
                        i = groupPathToken.countTokens();
                        allowed = true;
                    }
                }
                
                //Note that if the user is not in the default admin group
                //and also not in any of the nodes found in the path
                //the user is not allowed to admin the system.
            }
            
            if( allowed )
            {
                LinkedList list = new LinkedList();
                HashMap map = new HashMap();
                
                //The user is allowed to add the user to the group
                Object facObj = cx.lookup("UserFacadeEJB");
                UserFacadeHome facadeHome = (UserFacadeHome)PortableRemoteObject.narrow( facObj, UserFacadeHome.class );
                UserFacade userFacade = facadeHome.create();
                boolean added = userFacade.addUserToGroup( userName, groupName );
                
                String message = "";
                list.add("admin_message_object");
                
                if( added )
                {
                    message = "The user <b>"+userName+"</b> was added to the group <b>"+groupName+"</b>";
                    putMessageToModel( message, AdminMessageObject.OK );
                    map.put("group_inside_model", groupName );
                    list.add("group_inside_model");
                    
                }else 
                {
                    message = "The user <b>"+userName+"</b> was not added to the group <b>"+groupName+ "</b> check the group name!";
                    putMessageToModel( message, AdminMessageObject.WARNING );
                }
                
                
                //Then we will notify the framework about the changed model.
                HashMap events = eventConfig.getEvents();
                EventConfig theEventConfig = (EventConfig)events.get("addUserToGroup");
                
                EventResponse eventResponse = new EventResponse( list, map, theEventConfig.getView() );
                return eventResponse;                
                
                
            }else {
                //The user is not allowed to add the user to the group
                //Let's create an error message for the user.
                //The user was not an Admin, so we must show the error page
                //for the user.
                LinkedList list = new LinkedList();
                list.add("admin_message_object");
                HashMap map = new HashMap();
                
                //Let's save the message to the http session, so that 
                //the model can load it from there
                 putMessageToModel("The current "+
                            "user is not allowed to add the user"+userName+
                            "to the user group " + groupName+". Only node administrators"+
                            "are allowed to add users to user groups", AdminMessageObject.ERROR);
                
                 //Then we will notify the framework about the changed model.
                HashMap events = eventConfig.getEvents();
                EventConfig theEventConfig = (EventConfig)events.get("listUserGroup");
                
                EventResponse eventResponse = new EventResponse( list, map,theEventConfig.getView() );
                return eventResponse;                
            }
            
        }catch( Exception e ){
            return createErrorResponse( e.toString(), ((Object)this).getClass().getName(), "listUserGroup" );
        }
    }
    

    
    /**
     * Event removes certain user from the user group
     *
     */
    public EventResponse removeUserFromGroup( String userName, String groupName )
    {
        try {

            userName.trim();
            groupName.trim();
            //First we will check that the current user is
            //authenticated user!
            FrameworkContext securityContext = getSecurityContext();
            EventProcessorConfig eventConfig = getEventProcessorConfig();
            
            if( !securityContext.isAuthenticated() )
            {
                //User is not authenticated!
                //Let's return a error!
                return createErrorResponse( "User not authenticated", "UserAdminProcessor", "addUser" );
            }
            
            //First we will check if the user is authenticated and is in admin group
            //In future we will check if the user is in the "path" admin group. Let's check
            //it!
            InitialContext cx = new InitialContext();
            Object bound = cx.lookup("UserGroupEJB");
            UserGroupHome groupHome = (UserGroupHome)PortableRemoteObject.narrow( bound, UserGroupHome.class );
            Collection groups = groupHome.findByGroupName( groupName );
            
            UserGroup userGroup = null;
            
            if( groups.size() == 1 )
            {
                Iterator itr = groups.iterator();
                userGroup = (UserGroup)itr.next();
            }
            else{
                //No group was found or there
                //was too many groups (requires configuration!)
                //Let's create a message for the user
                LinkedList list = new LinkedList();
                list.add("admin_message_object");
                HashMap map = new HashMap();
                
                //Let's save the message to the http session, so that 
                //the model can load it from there
                AdminMessageObject admObj = new AdminMessageObject("The user group "+groupName+" was not found");
                HttpSession httpSession = getHttpSession();
                httpSession.setAttribute( AdminMessageObject.CONTEXT_NAME, admObj );
                
                HashMap events = eventConfig.getEvents();
                EventConfig theEventConfig = (EventConfig)events.get("removeUserFromGroup");
                
                EventResponse eventResponse = new EventResponse( list, map, theEventConfig.getView() );
                return eventResponse;                
            }
            
            //Ok! we found the user group. Now we will check if the user
            //has rights to add the user to the group!
            String groupPath = userGroup.getGroupPath();
            boolean allowed = false;

            //First we will check if the user is in default admin group.
            //If the user is in the group the event is automatically
            //allowed.
            FrameworkGroupPrincipal adminGroup = getDefaultAdminGroup();
            if( securityContext.isInGroup( adminGroup ) ){
                allowed = true;
            }else{
            
                //The user was not in the default admin group.
                //Now we must check the user groups path and the
                //try to figure is the user as admin in some of 
                //the nodes. We will iterate all admin groups node by
                //node and check if we found the user.
                StringTokenizer groupPathToken = new StringTokenizer( groupPath, "/" );
                String workPath = "/";
                
                for( int i = 0; i < groupPathToken.countTokens(); i++ )
                {
                    workPath += groupPathToken.nextToken();
                    FrameworkGroupPrincipal nodeAdminGroup = getAdminGroup( workPath );
                    if( nodeAdminGroup == null )
                        return createErrorResponse( "The node "+groupPath+" was not configurated propertly", ((Object)this).getClass().getName(), "removeUserFromGroup" );
                    
                    if( securityContext.isInGroup( nodeAdminGroup ) )
                    {
                        //Jep user is in the group. It is no 
                        //idea to check the rest of the node's 
                        //admin groups (they are childs for the current node)
                        i = groupPathToken.countTokens();
                        allowed = true;
                    }
                }
                
                //Note that if the user is not in the default admin group
                //and also not in any of the nodes found in the path
                //the user is not allowed to admin the system.
            }
            
            if( allowed )
            {
                //The user is allowed to add the user to the group
                Object facObj = cx.lookup("UserFacadeEJB");
                UserFacadeHome facadeHome = (UserFacadeHome)PortableRemoteObject.narrow( facObj, UserFacadeHome.class );
                UserFacade userFacade = facadeHome.create();
                boolean added = userFacade.removeUserFromGroup( userName, groupName );
                
                String message = "";
                if( added ){
                    message = "The user <b>"+userName+"</b> was removed from <b>"+groupName+"</b>";
                }else {
                    message = "The user <b>"+userName+"</b> was not removed from the group <b>"+groupName+ "</b>. The user was not found in it!";
                }
                
                //The user was not an Admin, so we must show the error page
                //for the user.
                LinkedList list = new LinkedList();
                HashMap map = new HashMap();
                
                list.add("admin_message_object");
                list.add("group_inside_model");
                map.put("group_inside_model", groupName );
                
                
                //Let's save the message to the http session, so that 
                //the model can load it from there
                AdminMessageObject admObj = new AdminMessageObject( message );
                HttpSession httpSession = getHttpSession();
                httpSession.setAttribute( AdminMessageObject.CONTEXT_NAME, admObj );
                
                //Then we will notify the framework about the changed model.
                HashMap events = eventConfig.getEvents();
                EventConfig theEventConfig = (EventConfig)events.get("removeUserFromGroup");
                
                EventResponse eventResponse = new EventResponse( list, map, theEventConfig.getView() );
                return eventResponse;                
                
                
            }else {
                //The user is not allowed to add the user to the group
                //Let's create an error message for the user.
                //The user was not an Admin, so we must show the error page
                //for the user.
                LinkedList list = new LinkedList();
                list.add("admin_message_object");
                HashMap map = new HashMap();
                
                //Let's save the message to the http session, so that 
                //the model can load it from there
                AdminMessageObject admObj = new AdminMessageObject("The current "+
                            "user is not allowed to remove the user"+userName+
                            "from the user group " + groupName+". Only node administrators"+
                            "are allowed to remove users from user groups");
                HttpSession httpSession = getHttpSession();
                httpSession.setAttribute( AdminMessageObject.CONTEXT_NAME, admObj );
                
                //Then we will notify the framework about the changed model.
                HashMap events = eventConfig.getEvents();
                EventConfig theEventConfig = (EventConfig)events.get("removeUserFromGroup");
                
                EventResponse eventResponse = new EventResponse( list, map, theEventConfig.getView() );
                return eventResponse;                
            }
            
        }catch( Exception e ){
            return createErrorResponse( e.toString(), ((Object)this).getClass().getName(), "removeUserFromGroup" );
        }
    }
    

    
    /**
     * Removes certain user group.
     */
    public EventResponse removeUserGroup( String groupName, String path )
    {
        try 
        {
            groupName.trim();
            //First we will check that the current user is
            //authenticated user!
            FrameworkContext securityContext = getSecurityContext();
            EventProcessorConfig eventConfig = getEventProcessorConfig();
            
            if( !securityContext.isAuthenticated() )
            {
                //User is not authenticated!
                //Let's return a error!
                return createErrorResponse( "User not authenticated", "UserAdminProcessor", "addUser" );
            }
            
            boolean allowed = false;

            //First we will check if the user is in default admin group.
            //If the user is in the group the event is automatically
            //allowed.
            FrameworkGroupPrincipal adminGroup = getDefaultAdminGroup();
            if( securityContext.isInGroup( adminGroup ) ){
                allowed = true;
            }else{
            
                //The user was not in the default admin group.
                //Now we must check the user groups path and the
                //try to figure is the user as admin in some of 
                //the nodes. We will iterate all admin groups node by
                //node and check if we found the user.
                StringTokenizer groupPathToken = new StringTokenizer( path, "/" );
                String workPath = "/";
                
                for( int i = 0; i < groupPathToken.countTokens(); i++ )
                {
                    workPath += groupPathToken.nextToken();
                    FrameworkGroupPrincipal nodeAdminGroup = getAdminGroup( workPath );
                    if( nodeAdminGroup == null )
                        return createErrorResponse( "The node "+workPath+" was not configurated propertly", ((Object)this).getClass().getName(), "addUserToGroup" );
                    
                    if( securityContext.isInGroup( nodeAdminGroup ) )
                    {
                        //Jep user is in the group. It is no 
                        //idea to check the rest of the node's 
                        //admin groups (they are childs for the current node)
                        i = groupPathToken.countTokens();
                        allowed = true;
                    }
                }
                
                //Note that if the user is not in the default admin group
                //and also not in any of the nodes found in the path
                //the user is not allowed to admin the system.
            }            
            
            //Let's check if the user was allowed to remove the
            //user group.
            if( allowed )
            {
                //The object required for the event response
                LinkedList list = new LinkedList();
                HashMap map = new HashMap();
                
                
                //Yep, the user belongs to the default admin
                //group. Lets create the user.
                InitialContext cx = new InitialContext();
                Object homeObj = cx.lookup("UserFacadeEJB");
                UserFacadeHome theHome = (UserFacadeHome)PortableRemoteObject.narrow( homeObj, UserFacadeHome.class );
                UserFacade userFacade = theHome.create();
                
                if( userFacade.existGroup( groupName ) )
                {
                    //Ok the group exist. Let's remove it!
                    boolean removed = userFacade.removeUserGroup( groupName );
                    if( removed )
                    {
                        list.add("admin_message_object");
                        list.add("group_list_model");
                        putMessageToModel("User group <b>"+groupName+"</b> removed succesfully.", AdminMessageObject.OK );
                    }
                    else
                    {
                        list.add("admin_message_object");
                        putMessageToModel("User group <b>"+groupName+"</b> was not found. Check the list to see all user groups.", AdminMessageObject.WARNING );
                    }
                }
                else
                {
                    list.add("admin_message_object");
                    putMessageToModel("User group <b>"+groupName+"</b> was not found. Check the list to see all user groups.", AdminMessageObject.WARNING );
                }
                
                HashMap events = eventConfig.getEvents();
                EventConfig theEventConfig = (EventConfig)events.get("removeUserGroup");
                EventResponse eventResponse = new EventResponse( list, map, theEventConfig.getView() );
                return eventResponse;
                
            }else {
                
                //The user is not allowed to create the user group under the
                //defined configuration node (unit)
                LinkedList list = new LinkedList();
                list.add("admin_message_object");
                HashMap map = new HashMap();
                
                //Let's save the message to the http session, so that 
                //the model can load it from there
                putMessageToModel("The current "+
                            "user is not allowed to remove the <b>" +groupName +
                            "</b> user group", AdminMessageObject.ERROR);
                
                //Then we will notify the framework about the changed model.
                HashMap events = eventConfig.getEvents();
                EventConfig theEventConfig = (EventConfig)events.get("removeUserGroup");
                
                EventResponse eventResponse = new EventResponse( list, map, theEventConfig.getView() );
                return eventResponse;               
                
            }
            
        }catch( Exception e ) {
            return createErrorResponse( e.toString(), ((Object)this).getClass().getName(), "removeUserGroup" );
        }
    }
    
    /**
     * Lists all users in certain user group
     *
     */
    public EventResponse listUserGroup( String groupName, String path ) {
        
        try 
        {
            groupName.trim();
            //First we will check that the current user is
            //authenticated user!
            FrameworkContext securityContext = getSecurityContext();
            EventProcessorConfig eventConfig = getEventProcessorConfig();
            
            if( !securityContext.isAuthenticated() )
            {
                //User is not authenticated!
                //Let's return a error!
                return createErrorResponse( "User not authenticated", "UserAdminProcessor", "addUser" );
            }
            
            boolean allowed = false;

            //First we will check if the user is in default admin group.
            //If the user is in the group the event is automatically
            //allowed.
            FrameworkGroupPrincipal adminGroup = getDefaultAdminGroup();
            if( securityContext.isInGroup( adminGroup ) ){
                allowed = true;
            }else{
            
                //The user was not in the default admin group.
                //Now we must check the user groups path and the
                //try to figure is the user as admin in some of 
                //the nodes. We will iterate all admin groups node by
                //node and check if we found the user.
                StringTokenizer groupPathToken = new StringTokenizer( path, "/" );
                String workPath = "/";
                
                for( int i = 0; i < groupPathToken.countTokens(); i++ )
                {
                    workPath += groupPathToken.nextToken();
                    FrameworkGroupPrincipal nodeAdminGroup = getAdminGroup( workPath );
                    if( nodeAdminGroup == null )
                        return createErrorResponse( "The node "+workPath+" was not configurated propertly", ((Object)this).getClass().getName(), "addUserToGroup" );
                    
                    if( securityContext.isInGroup( nodeAdminGroup ) )
                    {
                        //Jep user is in the group. It is no 
                        //idea to check the rest of the node's 
                        //admin groups (they are childs for the current node)
                        i = groupPathToken.countTokens();
                        allowed = true;
                    }
                }
                
                //Note that if the user is not in the default admin group
                //and also not in any of the nodes found in the path
                //the user is not allowed to admin the system.
            }            
            
            //Let's check if the user is allowed to list users in the
            //user group
            if( allowed )
            {
                LinkedList list = new LinkedList();
                HashMap map = new HashMap();
                
                InitialContext cx = new InitialContext();
                Object bound = cx.lookup("UserGroupEJB");
                UserGroupHome groupHome = (UserGroupHome)PortableRemoteObject.narrow( bound, UserGroupHome.class );
                Collection groups = groupHome.findByGroupName( groupName );
                
                Iterator itr = groups.iterator();
                
                if( groups.isEmpty() )
                {
                    //There was no user group!
                    list.add("admin_message_object");
                    putMessageToModel("User group <b>"+groupName+"</b> doesn't exist", AdminMessageObject.WARNING );
                }
                
                UserGroup userGroup = (UserGroup)itr.next();
                String groupId = userGroup.getGroupName();
                
                map.put("group_inside_model", groupId );
                list.add("group_inside_model");
                
                //Then we will notify the framework about the changed model.
                HashMap events = eventConfig.getEvents();
                EventConfig theEventConfig = (EventConfig)events.get("listUserGroup");
                EventResponse eventResponse = new EventResponse( list, map, theEventConfig.getView() );
                return eventResponse;                 
                
            }else {
                
                //The user is not allowed to create the user group under the
                //defined configuration node (unit)
                LinkedList list = new LinkedList();
                list.add("admin_message_object");
                HashMap map = new HashMap();
                
                //Let's save the message to the http session, so that 
                //the model can load it from there
                putMessageToModel("The current user is not allowed to list users in the <b>" +groupName +
                            "</b> user group", AdminMessageObject.ERROR);
                
                //Then we will notify the framework about the changed model.
                EventConfig theEventConfig = getEventConfig("listUserGroup");
                
                EventResponse eventResponse = new EventResponse( list, map, theEventConfig.getView() );
                return eventResponse;               
                
            }
            
        }catch( Exception e ){
            return createErrorResponse( e.toString(), ((Object)this).getClass().getName(), "listUserGroup" );
        }
    }

    /*
     *
     *
     */
    public EventResponse listUserGroups() 
    {
        EventConfig theEventConfig =  getEventConfig( "listUserGroups" );
        EventResponse eventResponse = new EventResponse( theEventConfig.getView() );
        return eventResponse;
    }
    
    public EventResponse listUsers() 
    {
        EventConfig theEventConfig =  getEventConfig( "listUsers" );
        EventResponse eventResponse = new EventResponse( theEventConfig.getView() );
        return eventResponse;
    }    
    
    /**
     * Puts a new message to the admin message model's http session part!
     */
    private void putMessageToModel( String message, int type )
    {
        //Let's save the message to the http session, so that 
        //the model can load it from there
        AdminMessageObject admObj = new AdminMessageObject( message,type );
        HttpSession httpSession = getHttpSession();
        httpSession.setAttribute( AdminMessageObject.CONTEXT_NAME, admObj );        
    }
    
    
}
